# Copyright 2022 by Jonas Lache <jonas.lache@ruhr-uni-bochum.de>
# SPDX-License-Identifier: GPL-3.0-or-later

# Note! This script only works for test data that only contains one set of stars ratings

import re
import csv
import statistics
from tkinter.filedialog import askopenfile, asksaveasfilename
# from os.path import exists

# Import file
text_file = askopenfile(filetypes=[("CSV text files", "*.csv")])

data = text_file.read()
text_file.close()

# =============================
# ====== Extract ratings ======
# =============================
# Assumes that in the test data there is only one STACK question item with a
# star rating, but it can contain any number of questions
pat_rat = re.compile('(?<=ratings\\\\"":).*(?=,\\\\""comment)')
result_rat = pat_rat.findall(data)

# Number of questions within the star rating:
num_ques = len(re.findall(",",result_rat[0]))+1

def ratingToList(rat): # Converts each set of ratings to a PY list
    list = []
    i=1
    while i<(len(rat)+1)/2:
        if int(rat[2*i-1])>0:
            list.append(int(rat[2*i-1]))
        else:
            list.append("")
        i+=1
    return list

result_rat_pylist = []
i=0
while i<len(result_rat):
    result_rat_pylist.append(ratingToList(result_rat[i]))
    i+=1

# =============================
# ====== Analyze ratings ======
# =============================
# Put all ratings from question j to a new list:
def getQuestionResults(j):
    ResultsQuestion_i = []
    i=0
    while i<len(result_rat_pylist):
        ResultsQuestion_i.append(result_rat_pylist[i][j])
        i+=1
    return ResultsQuestion_i

# Define a list that contains a list for each question, which then
# contains all results for this question:
ResultsByQuestion = []
i=0
while i<num_ques:
    ResultsByQuestion.append(getQuestionResults(i))
    i+=1

# Remove all elements '' from the lists (copy to new list)
ResultsByQuestion_cleaned = []
i=0
while i<num_ques:
    ResultsByQuestion_cleaned.append([x for x in ResultsByQuestion[i] if x!=""])
    i+=1

# Compute AM for each question:
AMList = []
i=0
while i<num_ques:
    AMList.append(statistics.mean(ResultsByQuestion_cleaned[i]))
    i+=1

# Compute Median for each question:
MedianList = []
i=0
while i<num_ques:
    MedianList.append(statistics.median(ResultsByQuestion_cleaned[i]))
    i+=1

# Compute SD for each question:
SDList = []
i=0
while i<num_ques:
    SDList.append(statistics.stdev(ResultsByQuestion_cleaned[i]))
    i+=1

# Compute frequency distribution:
def howOften(k,L):
    return len([x for x in L if x==k])

distributionList = []
i=0
while i<num_ques:
    distributionList_qi = []
    j=1
    while j<6:
        distributionList_qi.append(howOften(j,ResultsByQuestion_cleaned[i]))
        j+=1
    distributionList.append(distributionList_qi)
    i+=1

# Compute relativ frequencies
# Sum of elements in a list:
def sumArr(arr):
    sum=0
    i=0
    while i<len(arr):
        sum+=arr[i]
        i+=1
    return sum

# Multiply list with a number:
def multArrNum(arr,num):
    list=[]
    for el in arr:
        list.append(num*el)
    return list

sumSubarr=0
distributionList_rel =[]
i=0
for L in distributionList:
    sumSubarr=sumArr(L)
    distributionList_rel.append(multArrNum(L,1/sumSubarr))


# ============================
# ===== Extract comments =====
# ============================
# Assumes that there is only one task with a star rating in the test data
pat_com = re.compile('(?<=comment\\\\"":\\\\"").*(?=\\\\"",\\\\""submitted)')
result_com = pat_com.findall(data)
number_com = len(result_com)-result_com.count("") # Number of non-empty comments

# ============================
# ====== Prepare output ======
# ============================
# Make from list [a,b,c] a list of lists [[a],[b],[c]]
def extractDigits(lst):
    return [[el] for el in lst]

result_list_rat = extractDigits(result_rat)
result_list_com = extractDigits(result_com)

# Bring the lists [[ratings1], [ratings2], ...] und [comment1, comment2, ...]
# to the form [rating1_q1, rating1_q2, ...,  comment1], [rating_2_q1, rating_2_q2, ..., comment2], ...]

def mergeLists():
    L=[]
    i=0
    while i<len(result_rat):
        L1=[]
        j=0
        while j<num_ques:
            L1.append(result_rat_pylist[i][j])
            j+=1
        L1.append(result_com[i])
        L.append(L1)
        i+=1
    return L

mergedList = mergeLists()

# Generate list with names of the rows
rownames = []
i=1
while i < num_ques+1:
    rownames.append("Question "+str(i))
    i+=1
rownames.append("Comment")

exportFile = asksaveasfilename(filetypes=[("CSV text files", "*.csv")])

# Write the results to a CSV file
with open(exportFile, 'w') as f:
    write = csv.writer(f)
    write.writerow(rownames)
    write.writerows(mergedList)

exportFileStats = exportFile.replace(".csv", "_stats.csv")

with open(exportFileStats, 'w') as f:
    write = csv.writer(f)
    write.writerow(["Parameter(s):","Value(s):"])
    write.writerows([
        ["Number of ratings:",len(result_rat)],
        ["Number of questions within each rating:",num_ques],
        ["Number of comments:",number_com],
        ["Arithmetic mean for each question:",AMList],
        ["Median for each question:",MedianList],
        ["Standard deviation for each question:",SDList],
        ["Absolute frequencies:", distributionList],
        ["Relative frequencies:",distributionList_rel]
    ])

# =============================
# ======= Print results =======
# =============================
""""
print("Here are the stats on your star rating:")
print("Number of ratings: " + str(len(result_rat)))
print("Number of questions within each rating: " + str(num_ques))
print("Number of comments: " + str(number_com))
print("The arithmetic mean for each of your questions is: " + str(AMList))
print("The median for each of your questions is: " + str(MedianList))
print("The standard derivation for each of your questions is: " + str(SDList))
print("Frequency distribution of the number of stars for each of your questions: "
	+ str(distributionList))
print("Relative frequency of the number of stars for each of your questions: "
	+ str(distributionList_rel))
"""

print("The result has been exported to "+exportFile)
print("The stats have been exported to "+exportFileStats)
